#---------------------------------------------------------------
# * VisualK-OS (v5.0)
#   - File: COMMS-debugger.py
#   - Version: 1.1
#   - Author: Kevin Slaughter
#   - Desc: COMMS Manager debugging script; Tests specific
#           functions outside of the main program.
#
#
#   [Revision History]
#   2019/07/11: v1.1 - Added selection menu and various
#                      test functions.
#   2019/06/04: v1.0 - File created with initial framework.
#---------------------------------------------------------------



# Imports
#---------
import os, pygame, random, sys, time, vkos_globals
from collections import OrderedDict
from COMMS_Packet import COMMS_Packet
from objCOMMSManager import objCOMMSManager
from VKOS_Constants import *



# Variables
#-----------
CM = None		# COMMS Manager object
screen = None		# PyGame window.



# Functions
#--------------------------------------------
# clearScreen()
# Desc: Clear the terminal, based on the amount
#       of rows returnd by os.popen().
#--------------------------------------------
def clearScreen():
	print ("\n" * int(os.popen("stty size", "r").read().split()[0]))
	return



# handleInput(e)
# Desc: Handles keyboard input.
#--------------------------------------------
# -        k: PyGame keyboard event.
# - commsObj: COMMS Manager instance.
#--------------------------------------------
def handleInput(k, commsObj):
	k = k.key				# Key to respond to.
	tmpPacket = COMMS_Packet(COMMS_NOP)	# Dummy instantiation to quell Python.

	# Misc. keys
	#------------
	if (k == pygame.K_ESCAPE):					# Escape = Exit.
		vkos_globals.addStatusMsg("[ESC] Exiting...", "*")
		sys.exit()
	elif (k == pygame.K_F9): testRandom(commsObj)			# F9: testRandom()
	elif (k == pygame.K_F10): testSequential(commsObj)		# F10: testSequential()
	elif (k == pygame.K_F11): testCycle(commsObj)			# F11: testCycle()



	# State modifiers (Paired by number + F(x) key).
	# [1 = L0 OFF, F1 = L0 ON; 2 = L1 OFF, F2 = L1 ON...]
	#-----------------------------------------------------
	elif (k == pygame.K_1):		# 1: Lamp #0 OFF
		tmpPacket = COMMS_Packet(COMMS_OP_FADE_OUT, LAMP_0)

	elif (k == pygame.K_F1):	# F1: Lamp #0 ON
		tmpPacket = COMMS_Packet(COMMS_OP_FADE_IN, LAMP_0)

	elif (k == pygame.K_2):		# 2: Lamp #1 OFF
		tmpPacket = COMMS_Packet(COMMS_OP_FADE_OUT, LAMP_1)

	elif (k == pygame.K_F2):	# F2: Lamp #1 ON
		tmpPacket = COMMS_Packet(COMMS_OP_FADE_IN, LAMP_1)

	elif (k == pygame.K_3):		# 3: Lamp #2 OFF
		tmpPacket = COMMS_Packet(COMMS_OP_FADE_OUT, LAMP_2)

	elif (k == pygame.K_F3):	# F3: Lamp #2 ON
		tmpPacket = COMMS_Packet(COMMS_OP_FADE_IN, LAMP_2)

	elif (k == pygame.K_4):		# 4: Lamp #3 OFF
		tmpPacket = COMMS_Packet(COMMS_OP_FADE_OUT, LAMP_3)

	elif (k == pygame.K_F4):	# F4: Lamp #3 ON
		tmpPacket = COMMS_Packet(COMMS_OP_FADE_IN, LAMP_3)

	elif (k == pygame.K_5):		# 5: Lamp #4 OFF
		tmpPacket = COMMS_Packet(COMMS_OP_FADE_OUT, LAMP_4)

	elif (k == pygame.K_F5):	# F5: Lamp #4 ON
		tmpPacket = COMMS_Packet(COMMS_OP_FADE_IN, LAMP_4)


	# Color modifiers.
	#------------------
	# Lamp #0: [Q, W, E]
	#--------------------
	elif (k == pygame.K_q):		# Q: Lamp #0 = Red
		tmpPacket = COMMS_Packet(COMMS_OP_SET_COLOR, LAMP_0)
		tmpPacket.addParam(CPACK_ARG_COLOR, [0, 255, 0])

	elif (k == pygame.K_w):		# W: Lamp #0 = Green
		tmpPacket = COMMS_Packet(COMMS_OP_SET_COLOR, LAMP_0)
		tmpPacket.addParam(CPACK_ARG_COLOR, [110, 255, 0])

	elif (k == pygame.K_e):		# E: Lamp #0 = Blue
		tmpPacket = COMMS_Packet(COMMS_OP_SET_COLOR, LAMP_0)
		tmpPacket.addParam(CPACK_ARG_COLOR, [220, 255, 0])


	# Lamp #1: [A, S, D]
	#--------------------
	elif (k == pygame.K_a):		# A: Lamp #1 = Red
		tmpPacket = COMMS_Packet(COMMS_OP_SET_COLOR, LAMP_1)
		tmpPacket.addParam(CPACK_ARG_COLOR, [0, 255, 0])

	elif (k == pygame.K_s):		# S: Lamp #0 = Green
		tmpPacket = COMMS_Packet(COMMS_OP_SET_COLOR, LAMP_1)
		tmpPacket.addParam(CPACK_ARG_COLOR, [110, 255, 0])

	elif (k == pygame.K_d):		# D: Lamp #0 = Blue
		tmpPacket = COMMS_Packet(COMMS_OP_SET_COLOR, LAMP_1)
		tmpPacket.addParam(CPACK_ARG_COLOR, [220, 255, 0])


	# Lamp #2: [Z, X, C]
	#--------------------
	elif (k == pygame.K_z):		# Z: Lamp #2 = Red
		tmpPacket = COMMS_Packet(COMMS_OP_SET_COLOR, LAMP_2)
		tmpPacket.addParam(CPACK_ARG_COLOR, [0, 255, 0])

	elif (k == pygame.K_x):		# X: Lamp #2 = Green
		tmpPacket = COMMS_Packet(COMMS_OP_SET_COLOR, LAMP_2)
		tmpPacket.addParam(CPACK_ARG_COLOR, [110, 255, 0])

	elif (k == pygame.K_c):		# C: Lamp #2 = Blue
		tmpPacket = COMMS_Packet(COMMS_OP_SET_COLOR, LAMP_2)
		tmpPacket.addParam(CPACK_ARG_COLOR, [220, 255, 0])


	# Lamp #3: [R, T, Y]
	#--------------------
	elif (k == pygame.K_r):		# R: Lamp #3 = Red
		tmpPacket = COMMS_Packet(COMMS_OP_SET_COLOR, LAMP_3)
		tmpPacket.addParam(CPACK_ARG_COLOR, [0, 255, 0])

	elif (k == pygame.K_t):		# T: Lamp #3 = Green
		tmpPacket = COMMS_Packet(COMMS_OP_SET_COLOR, LAMP_3)
		tmpPacket.addParam(CPACK_ARG_COLOR, [110, 255, 0])

	elif (k == pygame.K_y):		# Y: Lamp #3 = Blue
		tmpPacket = COMMS_Packet(COMMS_OP_SET_COLOR, LAMP_3)
		tmpPacket.addParam(CPACK_ARG_COLOR, [220, 255, 0])


	# Lamp #4: [F, G, H]
	#--------------------
	elif (k == pygame.K_f):		# F: Lamp #4 = Red
		tmpPacket = COMMS_Packet(COMMS_OP_SET_COLOR, LAMP_4)
		tmpPacket.addParam(CPACK_ARG_COLOR, [0, 255, 0])

	elif (k == pygame.K_g):		# G: Lamp #4 = Green
		tmpPacket = COMMS_Packet(COMMS_OP_SET_COLOR, LAMP_4)
		tmpPacket.addParam(CPACK_ARG_COLOR, [110, 255, 0])

	elif (k == pygame.K_h):		# H: Lamp #4 = Blue
		tmpPacket = COMMS_Packet(COMMS_OP_SET_COLOR, LAMP_4)
		tmpPacket.addParam(CPACK_ARG_COLOR, [220, 255, 0])
	else:
		if ((k > 33) and (k < 126)): errorKey = chr(k)
		else: errorKey = str(k)
		print "[ERROR: Unmapped key: '" + errorKey + "]"
		return


	# Send packet
	#-------------
	if (tmpPacket):
		commsObj.devices["FBA"].sendPacket(tmpPacket)
		del tmpPacket
	return



# showHeader()
# Desc: Displays a program info header.
#--------------------------------------------
def showHeader():
	print ("-" * 45)
	print ("VisualK-OS COMMS-debugger Utility")
	print ("-" * 45)
	return



# showMenu()
# Desc: Displays the console debugging menu.
#--------------------------------------------
def showMenu():
	clearScreen()
	showHeader()
	print "Lamp #0: [F1 = ON  /  1 = OFF] [Q = Red   -   W = Green   -   E = Blue]"
	print "Lamp #1: [F1 = ON  /  1 = OFF] [A = Red   -   S = Green   -   D = Blue]"
	print "Lamp #2: [F1 = ON  /  1 = OFF] [Z = Red   -   X = Green   -   C = Blue]"
	print "Lamp #3: [F1 = ON  /  1 = OFF] [R = Red   -   T = Green   -   Y = Blue]"
	print "Lamp #4: [F1 = ON  /  1 = OFF] [F = Red   -   G = Green   -   H = Blue]"
	print "\nSpecial Keys:"
	print "    [F9] testRandom()"
	print "   [F10] testSequential()"
	print "   [F11] testCycle()"
	return



# testCycle()
# Desc: Cycles through the full
#       0~360 HSV color range.
#---------------------------------------
# - commsObj: COMMS_Manager instance.
#---------------------------------------
def testCycle(commsObj):
	brightnessPacket = COMMS_Packet(COMMS_OP_SET_BRIGHTNESS, ALL_LAMPS, [CPACK_ARG_BYTE, 255])	# Enable lamps.
	commsObj.devices["FBA"].sendPacket(brightnessPacket)

	# Cycle through all 360 HSV colors.
	for i in range(0, 5):
		for h in range(0, 361, 5):
			colorPacket = COMMS_Packet(COMMS_OP_SET_COLOR, ALL_LAMPS, [CPACK_ARG_COLOR, [h, 255, 255]])
			print "h = " + str(h)
			commsObj.devices["FBA"].sendPacket(colorPacket)
			time.sleep(0.25)
			del colorPacket

			print "   - Brightness loop."
			for i in range(0, 2):
				for b in range(0, 255, 15):
					brightness = (255-b) if (i) else b
					bPacket = COMMS_Packet(COMMS_OP_SET_BRIGHTNESS, ALL_LAMPS, [CPACK_ARG_BYTE, brightness])
					commsObj.devices["FBA"].sendPacket(bPacket)
					time.sleep(0.25)
		time.sleep(1.0)

	# Turn off all lamps after test cycles.
	commsObj.devices["FBA"].sendPacket(COMMS_Packet(COMMS_OP_OFF, ALL_LAMPS))
	return



# testRandom(commsObj)
# Desc: Flashes 3 random lamps of
#       randomly-generated colors.
#---------------------------------------
# - commsObj: COMMS_Manager instance.
#---------------------------------------
def testRandom(commsObj):
	vkos_globals.addStatusMsg("Running random color/lamp test.", "*")
	for i in range(0, 16):
		# Generate random colors and lamp indices.
		#------------------------------------------
		iRndColor = [random.randrange(0, 100), random.randrange(100, 220), random.randrange(221, 360)]
		iRndLamp = [random.randrange(100, 103), random.randrange(100, 102), random.randrange(100, 104)]

		# Ensure we get 3 different lamps
		#---------------------------------
		bRandomGen = True
		while (bRandomGen):
			if (iRndLamp[0] == iRndLamp[1]): iRndLamp[0] = random.randrange(100, 105)
			if (iRndLamp[0] == iRndLamp[2]): iRndLamp[0] = random.randrange(100, 105)
			if (iRndLamp[1] == iRndLamp[2]): iRndLamp[1] = random.randrange(100, 105)
			bRandomGen = not ((iRndLamp[0] <> iRndLamp[1]) and (iRndLamp[0] <> iRndLamp[2]) and (iRndLamp[1] <> iRndLamp[2]))

		# Set colors
		#------------
		for c in range(0, 3):
			t = COMMS_Packet(COMMS_OP_SET_COLOR, iRndLamp[c])
			t.setColor([iRndColor[c], 255, 0])
			commsObj.devices["FBA"].sendPacket(t)
			t.clearPacket(); CM.processIO()

		# Fade in
		#---------
		for c in range(0, 3):
			t.setCmd(COMMS_OP_FADE_IN, iRndLamp[c])
			commsObj.devices["FBA"].sendPacket(t)
			t.clearPacket(); time.sleep(0.2)

		# Fade out
		#----------
		for c in range(0, 3):
			t.setCmd(COMMS_OP_FADE_OUT, iRndLamp[c])
			commsObj.devices["FBA"].sendPacket(t)
			t.clearPacket(); time.sleep(0.3)

	vkos_globals.addStatusMsg("Finished random color/lamp test.", "*", 1)
	return



# testSequential(commsObj)
# Desc: Sets lamps with a scene and
#       sequentially fades them in/out.
#---------------------------------------
# - commsObj: COMMS-Manager instance.
#---------------------------------------
def testSequential(commsObj):
	vkos_globals.addStatusMsg("Running sequential test (Individual lamps 3x, then all simultaneously).", "*")

	# Set up packet array.
	tmpPacket = {"L0-OFF"  : COMMS_Packet(COMMS_OP_FADE_OUT, LAMP_0), "L0-ON" : COMMS_Packet(COMMS_OP_FADE_IN, LAMP_0),
                     "L1-OFF"  : COMMS_Packet(COMMS_OP_FADE_OUT, LAMP_1), "L1-ON" : COMMS_Packet(COMMS_OP_FADE_IN, LAMP_1),
                     "L2-OFF"  : COMMS_Packet(COMMS_OP_FADE_OUT, LAMP_2), "L2-ON" : COMMS_Packet(COMMS_OP_FADE_IN, LAMP_2),
                     "L3-OFF"  : COMMS_Packet(COMMS_OP_FADE_OUT, LAMP_3), "L3-ON" : COMMS_Packet(COMMS_OP_FADE_IN, LAMP_3),
                     "L4-OFF"  : COMMS_Packet(COMMS_OP_FADE_OUT, LAMP_4), "L4-ON" : COMMS_Packet(COMMS_OP_FADE_IN, LAMP_4),
                     "ALL-OFF" : COMMS_Packet(COMMS_OP_FADE_OUT, FBA_BANK_A), "ALL-ON"  : COMMS_Packet(COMMS_OP_FADE_IN, FBA_BANK_A),
                     "SET-SCENE0" : COMMS_Packet(COMMS_OP_SET_SCENE, SCENE_0), "SET-SCENE1" : COMMS_Packet(COMMS_OP_SET_SCENE, SCENE_1),
		     "L0-SPECIAL" : COMMS_Packet(COMMS_OP_SET_COLOR, LAMP_0)}

	# Set lamp colors.
	commsObj.devices["FBA"].sendPacket(tmpPacket["SET-SCENE0"]); time.sleep(0.1)

	# Sequentially fade each lamp in and out (3x).
	for c in range(0, 3):
		# Fade in each lamp.
		for i in range(0, 5):
			strLamp = "L" + str(i) + "-ON"
			commsObj.devices["FBA"].sendPacket(tmpPacket[strLamp]); time.sleep(0.1)

		# Fade out.
		for i in range(0, 5):
			strLamp = "L" + str(i) + "-OFF"
			commsObj.devices["FBA"].sendPacket(tmpPacket[strLamp]); time.sleep(0.1)

	# Fade all lamps in and out, simultaneously.
	for i in range(0, 3):
		commsObj.devices["FBA"].sendPacket(tmpPacket["ALL-ON"]); time.sleep(0.05)
		commsObj.devices["FBA"].sendPacket(tmpPacket["ALL-OFF"]); time.sleep(0.05)

	vkos_globals.addStatusMsg("Sequential test finished.", "*", 1)
	return



# Main
#------
if __name__ == "__main__":
	clearScreen()
	showHeader()

	# Initialize COMMS subsystem.
	#-----------------------------
	CM = objCOMMSManager()
	if (CM):
		vkos_globals.addStatusMsg("Initialized COMMS Manager object.", "*")

		# Set up PyGame / BG image.
		#---------------------------
		pygame.init()
		pyClock = pygame.time.Clock()
		screen = pygame.display.set_mode([800,600]); pygame.display.set_caption("VisualK-OS (v5.0):: COMMS Debugger")
		bgImage = pygame.image.load("img/_debugger.png").convert()

		# Fade in BG image.
		#-------------------
		for f in range(0, 128, 16):
			bgImage.set_alpha(f)				# Adjust opacity.
			screen.fill([0, 0, 0], [0, 0, 800, 600])	# Blank screen.
			screen.blit(bgImage, [0, 0])			# Blit image.
			pygame.display.update()
			pyClock.tick(60)
		bgImage.set_alpha(256); pygame.display.update()

		# Display status message.
		#-------------------------
		textFont = pygame.font.SysFont("Arial", 32)
		screen.blit(textFont.render("Handshaking (FBA)...", False, [0, 0, 255]), [200, 250])
		pygame.display.update(); time.sleep(0.25)

		# Connect/Handshake with devices.
		#---------------------------------
		CM.connectDevice("FBA")
		if (CM.isDeviceConnected("FBA")):
			vkos_globals.printLog("Successfully synced with FBA!", "*")

			# Finish BG image fade in.
			#--------------------------
			for f in range(128, 256, 16):
				bgImage.set_alpha(f)				# Adjust opacity.
				screen.fill([0, 0, 0], [0, 0, 800, 600])	# Blank screen.
				screen.blit(bgImage, [0, 0])			# Blit image.
				pygame.display.update()
				pyClock.tick(60)
		else:
			# Dim background image.
			#-----------------------
			for a in range(256, 64, -16):
				bgImage.set_alpha(a)				# Adjust opacity.
				screen.fill([0, 0, 0], [0, 0, 800, 600])	# Blank screen.
				screen.blit(bgImage, [0, 0])			# Blit image.
				pygame.display.update()				# Refresh screen.
				pyClock.tick(60)

			# Draw error text.
			#------------------
			textFont = pygame.font.SysFont("Arial", 32)
			screen.blit(textFont.render("Cannot open serial port (FBA).", False, [0, 0, 255]), [200, 250])
			pygame.display.update(); vkos_globals.addStatusMsg("Could not establish link with FBA... Exiting.", "ERROR")
			time.sleep(1)
			sys.exit()


		# Main loop
		#-----------
		showMenu()			# Display debug menu
		bActive = True
		while (bActive):
			CM.processIO()		# Check for incoming data from each device.

			# Check input.
			#--------------
			events = pygame.event.get()
			for event in events:
				if (event.type == pygame.KEYDOWN): handleInput(event, CM)

			# Refresh screen.
			#-----------------
			screen.blit(bgImage, [0, 0])
			pygame.display.update()
			pyClock.tick(60)	# Limit to 60FPS.

		sys.exit()	# Exit when main loop is completed.
	else:
		# Could not instantiate COMMS Manager.
		vkos_globals.addStatusMsg("Error creating COMMS-Manager object... Exiting.", "ERROR")
		sys.exit()
