#---------------------------------------------------------------
# * VisualK-OS (v5.0)
#   - File: UIE_base.py
#   - Version: 1.4
#   - Author: Kevin Slaughter
#   - Desc: UI Element base class, inherited by all UIEs.
#
#
#   [Revision History]
#   2019/09/23: v1.4 - Added placeholder handleMouse() function.
#   2019/02/09: v1.3 - Commenting/Documentation standardization.
#   2019/02/04: v1.2 - General housekeeping. Changed colors
#                      into a dictionary, for simplification.
#   2019/01/01: v1.1 - Changed naming convention to UIE_Base.
#   2018/12/27: v1.0 - File created with initial framework.
#---------------------------------------------------------------



# Imports
#---------
import pygame, vkos_globals



# Class Object
#--------------
class UIE_Base(object):
	# Initialization
	#----------------
	def __init__(self, dispRect, aLoc):
		# Set themed colors.
		#--------------------
		self.colors = {"BG"    : vkos_globals.TC_BG_SECTION,
		               "FG"    : vkos_globals.TC_FG_SECTION,
		               "Frame" : vkos_globals.TC_FRAME}

		# Set up general UIE base.
		#--------------------------
		self.aLoc = [aLoc[0], aLoc[1], dispRect[2], dispRect[3]]	# Absolute screen coordinates.
		self.rLoc = [dispRect[0], dispRect[1]]				# Host-relative coordinates.
		self.size = [dispRect[2], dispRect[3]]				# Store dimensions.


		# Set general parameters.
		#-------------------------
		self.enabled = True					# Enables/Disables input for the UIE.
		self.fontMain = vkos_globals.SM.fonts["Main"]		# Main font for text.
		self.focus = False					# Focus state, for keyboard usage.
		self.mouseDown = False					# Mouse button state indicator.
		self.mouseButton = -1					# Mouse button currently being held down.
		self.name = ""						# UIE identifier.
		self.owner = ""						# Section the UIE is hosted within.
		self.redraw = True					# Flag for indicating a redraw is needed.
		self.surfMain = pygame.Surface(self.size)		# Create main drawing surface.
		self.type = "XYGOTE"					# UIE object type (BUTTON, SLIDER, CHECKBOX).
		return



	# Shared Functions
	#-------------------------------------------------------
	# getLoc()
	# Desc: Returns a list with coordinates relative to the UIE's host.
	#-------------------------------------------------------
	def getLoc(self):
		return [self.rLoc[0], self.rLoc[1], self.size[0], self.size[1]]


	#-------------------------------------------------------
	# getAbsRect()
	# Desc: Returns a list with absolute screen
	#       position/dimensions.
	#-------------------------------------------------------
	def getAbsRect(self):
		return [self.aLoc[0], self.aLoc[1], self.size[0], self.size[1]]


	#-------------------------------------------------------
	# getRect()
	# Desc: Returns a list that encompasses the entire UIE.
	#-------------------------------------------------------
	def getRect(self):
		return [0, 0, self.size[0], self.size[1]]



	#-------------------------------------------------------
	# handleMouse(e)
	# Desc: Placeholder for UIE_Base-derived objects that
	#       don't need to handle mouse input directly,
	#       such as buttons.
	#-------------------------------------------------------
	# - e: Event packet.
	#-------------------------------------------------------
	def handleMouse(self, e):
		return
