import serial, time
from VKOS_Constants import *



# doHandshake(serConnection):
# Desc: Setup and sync device with main system.
#-----------------------------------------------
def doHandshake(serConnection):
    if (bActive):
        serConnection.flushInput(); serConnection.flushOutput()     # Clear serial I/O buffers.
        bPacketStarted = False; bPacketEnded = False; bGotID = False
        bHandshook = False; iSyncAcks = 0
        global bActive
        while (bActive):
            if (serConnection.inWaiting() > 0):
                inByte = serConnection.read(1)
                
                # Analyze.
                #----------
                # Packet start
                if (ord(inByte) == CPACK_START):
                    print "[*] Received start of packet."
                    if not (bPacketStarted):
                        if not (bPacketEnded):
                            bPacketStarted = True
                        else:
                            bPacketStarted = True
                            bPacketEnded = False        # New packet! Clear any old info.
                    else:
                        print "Something went wrong...  Packet header/end out of order.\nExiting."
                        quit()
                
                # General acknowledgment
                if (ord(inByte) == COMMS_ACK):
                    if (bHandshook): iSyncAcks += 1
            
                # Connection request.
                if (ord(inByte) == COMMS_OP_CONNECT):
                    print "   - Received connection request."
                    bGotID = False
            
                # Device ID.
                if (ord(inByte) == COMMS_DEVICE_ID):
                    if ((bPacketStarted) and not (bPacketEnded)):
                        while not (bGotID):     # Block until we get an ID.
                            if (serConnection.inWaiting() > 0):
                                iID = serConnection.read(1)
                                if (ord(iID) == ID_FBA):
                                    print "   - ID = FBA"
                                    bGotID = True
                                elif (ord(iID) == ID_THOR):
                                    print "   - ID = THOR"
                                    bGotID = True
                            time.sleep(0.1)
                        
                        # Send acknowledgment packet.
                        serConnection.write(chr(CPACK_START))
                        serConnection.write(chr(COMMS_ACK))
                        serConnection.write(chr(CPACK_END))
                
                # Packet end.
                if (ord(inByte) == CPACK_END):
                    print "[*] Received end of packet."
                    if ((bPacketStarted) and (bGotID)):
                        print "[*] Handshake successful!\n"
                        bPacketEnded = True;    bHandshook = True
                        bPacketStarted = False; bGotID = False
                    elif ((bPacketStarted) and (bHandshook)):
                            # Check for sync-confirming double acknowledgment from Arduino.
                        if (iSyncAcks == 2):
                            print "[*] Arduino is synced with main system!"
                            bActive = False     # Exit function.
                        else:
                            print "[*] Did not get double ACK from Arduino. Sync error!\nExiting."
                            quit()
                    else:       # Missing data, got packet start/end, but no ID.
                        print "[!] Something went wrong... Got packet start/end, but no ID.\nExiting."
                        quit()
            time.sleep(0.05)
    return



# MAIN CODE
#-----------
if (__name__ == "__main__"):
    serDemo = serial.Serial('/dev/vkos/FBA', 115200)
    global bActive
    bActive = bool(serDemo)
    doHandshake(serDemo)
    
    bPacketStarted = False
    bPacketEnded = False
    packetCmds = {}

    while (True):
        if (serDemo.inWaiting() > 0):
            inChar = serDemo.read(1)
            
            # Packet start
            if (ord(inChar) == CPACK_START):
                if (not (bPacketStarted) and not (bPacketEnded)):
                    # New packet!
                    bPacketStarted = True
                    print "[*] Packet started."
                else:       # Something went wrong; Starting a new packet while within one..?
                    print "[!] ERROR: Got new packet byte while within an existing packet."
            
            if (ord(inChar) == COMMS_OP_FADE_IN_BANK):
                packetCmds.update({"cmd0" : "FADE IN"})
                print "   - Added command: Fade In"

            if (ord(inChar) == CPACK_ARG_CONST):
                inChar = serDemo.read(1)        # Get next byte.
                if (ord(inChar) == FBA_BANK_A):
                    packetCmds.update({"arg0" : int(FBA_BANK_A)})
                    print "   - Added arg: FBA_BANK_A"

            if (ord(inChar) == CPACK_ARG_LITERAL):
                inNumber = serDemo.parseInt()
                print "   - Got literal number: '" + str(inNumber) + "'"

            # Packet end
            if (ord(inChar) == CPACK_END):
                if ((bPacketStarted) and not (bPacketEnded)):
                    bPacketEnded = True
                    print "[*] Packet ended."
                else:
                    print "[!] ERROR"


            # Analyze/execute packet.
            if (bPacketEnded):
                print "\n\n*** Packet info:"
                print "-----------------"
                print "   - packetCmds = " + str(packetCmds)
                print "-----------------"

