#---------------------------------------------------------------
# * VisualK-OS (v5.0)
#   - File: vkos_cc.py
#   - Version: 1.6
#   - Author: Kevin Slaughter
#   - Desc: Main VisualK-OS Engine
#           Program must be started from this file.
#
#
#   [Revision History]
#   2019/09/01: v1.6 - Simplification of command line arguments.
#   2019/03/21: v1.5 - General clean-up and code simplification.
#   2019/02/09: v1.4 - Commenting/Documentation standardization.
#   2019/02/04: v1.3 - General/Numerous changes/bugfixes/etc.
#   2019/01/14: v1.2 - Added basic COMMS management module.
#   2018/12/25: v1.1 - Cleaned up imports and globals usage.
#   2018/11/16: v1.0 - File created with initial framework.
#---------------------------------------------------------------



# Imports
#---------
import os, sys, pygame, time, vkos_globals
from objCOMMSManager import objCOMMSManager
from objPopup import objPopup
from objScreenManager import objScreenManager



# Main Program
#--------------
if __name__ == "__main__":
	# Clear the screen and display branding/version info.
	#-----------------------------------------------------
	print(("\n" * 16) + str(vkos_globals.PROGTITLE + " (v" + vkos_globals.PROGVERSION + ")"))
	print("-" * (len(vkos_globals.PROGTITLE) + len(vkos_globals.PROGVERSION) + 4))

	# Process any command line arguments.
	#-------------------------------------
	displaySplashScreen = True 	# Splash screen is displayed by default.
	if (len(sys.argv)>1):
		for arg in sys.argv:
			if ("--emu" in arg):			# --emu: Emulate slave devices (Disables COMMS I/O)
				vkos_globals.emulateDevices = True
				vkos_globals.printLog("\"--emu\" detected... COMMS I/O disabled.", "arg")
			if ("--no-splash" in arg):		# --no-splash: Disables splash screen
				displaySplashScreen = False
				vkos_globals.printLog("\"--no-splash\" detected... Splash Screen disabled.", "arg")
			if ("--mouse-info" in arg):	# --show-mouse-info: Display mouse position.
				vkos_globals.showMouseInfo = True
				vkos_globals.printLog("\"--mouse-info\" detected... Mouse coordinates will be displayed.", "arg")
		print("-" * (len(vkos_globals.PROGTITLE) + len(vkos_globals.PROGVERSION) + 4))


	# Initialize display and show splash screen.
	#--------------------------------------------
	SCREENMAN = objScreenManager()
	if (SCREENMAN):
		vkos_globals.SM = SCREENMAN		# (Hacky) Global access to the ScreenManager class.
		if (displaySplashScreen): SCREENMAN.showSplashScreen()
		SCREENMAN.showDesktop(); SCREENMAN.updateDisplay()
	else:
		vkos_globals.printLog("Error initializing ScreenManager. Cannot continue; Exiting.", "ERROR")
		quit(); sys.exit()

	# Initalize COMMS Manager object.
	#---------------------------------
	vkos_globals.printLog("Initializing COMMS module.", "*", 1)
	COMMSMAN = objCOMMSManager()
	if (COMMSMAN): vkos_globals.CM = COMMSMAN	# (Hacky) Global access to the COMMSManager class.
	else:
		vkos_globals.printLog("Error initializing COMMS Management. No reason to continue; Exiting.", "ERROR")
		quit(); sys.exit()


	# Main loop.
	#------------
	while (SCREENMAN.desktop.active):
		COMMSMAN.processIO()		# Process any incoming serial data.
		SCREENMAN.updateDisplay()	# Update timers/UIEs and redraw as needed.
		SCREENMAN.clock.tick(60)	# Limit to 60FPS.

	# When the showDesktop() function exits its loop, all device lamps are turned off
	# in the objDesktop class and we fall back out here to exit with a flourish.
	vkos_globals.printLog("Desktop no longer active... Exiting normally.", "*")
	SCREENMAN.doPowerOut()		# Schwupp!
